==> playship-carroms-1.1.5.tgz: package/src/logic/hangman.js <==
const oddUtils = require('../utils/oddUtils');
const logger = oddUtils.getLogger();
const genericFetchUtils =
    require('../utils/fillAndFetchUtils').genericFetchUtils;

const LIST_OF_CENTER_POS = oddUtils.getListOfCenteringBoundaries();
const ALLOWED_MARGIN = oddUtils.allowedMargin();
const CASES_SNAPSHOT_CHECK = oddUtils.casesForSnapshotMatchCheck();

module.exports.guillotine = function(game, coinsToPutBack, cb) {
  const gameKey = game.getId();
  // update collision map in neutrino
  game.updateCollisionMap((err, _) => {
    if (err)
      logger.error({key: gameKey}, 'Error updating collision map.');

    // get list of positions to place coins
    centeringPositions(game, coinsToPutBack.length).then(res => {
      // now update positions and relay the force update to client
      forceUpdatePositions(game, coinsToPutBack, res, cb);
    }).catch(error => {
      cb(error, null);
    });
  });
};

module.exports.trialByPosition = function(game, gameObjectsToCheck) {
  const gameKey = game.getId();
  // for now passing all position validations
  for (let bGO of gameObjectsToCheck) {
    let aGO = game.gameObjectPositionByIds(bGO.id)[bGO.id];
    logger.trace({key: gameKey}, 'printing before validation', aGO, bGO);
    // x
    let diff;
    diff = Math.abs(aGO.x - bGO.position.x);
    if (diff > ALLOWED_MARGIN.ALL_OK.x)
      return CASES_SNAPSHOT_CHECK.ABANDON_MATCH;
    // y
    diff = Math.abs(aGO.y - bGO.position.y);
    if (diff > ALLOWED_MARGIN.ALL_OK.y)
      return CASES_SNAPSHOT_CHECK.ABANDON_MATCH;
    // z
    diff = Math.abs(aGO.z - bGO.position.z);
    if (diff > ALLOWED_MARGIN.ALL_OK.z)
      return CASES_SNAPSHOT_CHECK.ABANDON_MATCH;
  }
  return CASES_SNAPSHOT_CHECK.ALL_OK;
};

const forceUpdatePositions = function(game, coinsToPutBack, posList, cb) {
  const gameKey = game.getId();
  logger.debug({key: gameKey}, 'PosList to put coins back is ', posList);
  // if no coin to put back just call the callback
  if (coinsToPutBack.length === 0) return cb(null, null);

  if (coinsToPutBack.length !== posList.length)
    logger.error({key: gameKey}, 'Poslist doesn\'t match with coins to put back list');

  let objTransList = [];
  // update coins position, velocity, force and rotation
  for (let i = 0; i < coinsToPutBack.length; i++) {
    let coin = coinsToPutBack[i];
    let posObj = posList[i];
    let objTrans = {
      id: coin.id,
      posX: posObj.posX,
      posY: posObj.posY,
      posZ: posObj.posZ,
    };
    objTransList.push(objTrans);
  }
  cb(null, () => {
    game.forceUpdateGameObjects(...objTransList);
  });
};

const centeringPositions = async function(game, coinsToCenterCount) {
  let coinsCentred = 0;
  let positionsToPush = [];
  let hand = genericFetchUtils.fetchHand(game);

  for (let i = 0, len = LIST_OF_CENTER_POS.length;
       i < len && coinsCentred < coinsToCenterCount; i++) {
    let posBoundary = LIST_OF_CENTER_POS[i];
    let res = await promisifyCollision(game, posBoundary);
    if (res.length === 0 || (res.length === 1 && res[0] === hand.id)) {
      // found position push it into list and increment count
      positionsToPush.push(posBoundary);
      coinsCentred++;
    }
  }
  return positionsToPush;
};

const promisifyCollision = function(game, boundary) {
  return new Promise((resolve, reject) => {
    game.collidedObjsWithBoundary(boundary, (err, res) => {
      if (err)
        return reject(err);
      resolve(res);
    });
  });
};

==> playship-carroms-1.1.9.tgz: package/src/logic/hangman.js <==
const oddUtils = require('../utils/oddUtils');
const logger = oddUtils.getLogger();
const genericFetchUtils =
    require('../utils/fillAndFetchUtils').genericFetchUtils;

const LIST_OF_CENTER_POS = oddUtils.getListOfCenteringBoundaries();
const ALLOWED_MARGIN = oddUtils.allowedMargin();
const CASES_SNAPSHOT_CHECK = oddUtils.casesForSnapshotMatchCheck();
const ALL_PROP_KEYS = oddUtils.getAllPropKeys();

module.exports.guillotine = function(game, coinsToPutBack, cb) {
  const gameKey = game.getId();
  // update collision map in neutrino
  game.updateCollisionMap((err, _) => {
    if (err)
      logger.error({key: gameKey}, 'Error updating collision map.');

    // get list of positions to place coins
    centeringPositions(game, coinsToPutBack.length).then(res => {
      // now update positions and relay the force update to client
      forceUpdatePositions(game, coinsToPutBack, res, cb);
    }).catch(error => {
      cb(error, null);
    });
  });
};

module.exports.trialByPosition = function(game, gameObjectsToCheck) {
  const gameKey = game.getId();
  let totCoins = genericFetchUtils.fetchGameEndScore(game) * 2 + 1;
  let coinsPotted = 0;
  for (let p of game.getAllPlayerIds()) coinsPotted += game.getPlayerScore(p);

  // for now passing all position validations
  let coinsMismatchCount = 0;
  for (let bGO of gameObjectsToCheck) {
    let aGO = game.gameObjectPositionByIds(bGO.id)[bGO.id];
    let gameObj = genericFetchUtils.fetchObjById(game, bGO.id);
    if (oddUtils.parseBoolean(gameObj.getProp(ALL_PROP_KEYS.IS_SCORED)))
      continue;
    logger.trace({key: gameKey}, 'printing before validation', aGO, bGO);
    // x
    let diff;
    diff = Math.abs(aGO.x - bGO.position.x);
    if (diff > ALLOWED_MARGIN.ALL_OK.x) {
      coinsMismatchCount++;
      logger.warn({key: gameKey}, 'Coin position mismatch found',
          JSON.stringify({serverO: aGO, clientO: bGO.position}));
      continue;
    }
    // y
    diff = Math.abs(aGO.y - bGO.position.y);
    if (diff > ALLOWED_MARGIN.ALL_OK.y) {
      coinsMismatchCount++;
      logger.warn({key: gameKey}, 'Coin position mismatch found',
          JSON.stringify({serverO: aGO, clientO: bGO.position}));
      continue;
    }
    // z
    diff = Math.abs(aGO.z - bGO.position.z);
    if (diff > ALLOWED_MARGIN.ALL_OK.z) {
      coinsMismatchCount++;
      logger.warn({key: gameKey}, 'Coin position mismatch found',
          JSON.stringify({serverO: aGO, clientO: bGO.position}));
      continue;
    }
    logger.trace({key: gameKey}, 'Validation passed for coin', bGO.id);
  }
  if (coinsMismatchCount > ((totCoins - coinsPotted) / 2)) {
    logger.warn({key: gameKey}, 'State mismatch has been found',
        {totCoins, coinsMismatchCount, coinsPotted});
    return CASES_SNAPSHOT_CHECK.ABANDON_MATCH;
  }
  return CASES_SNAPSHOT_CHECK.ALL_OK;
};

const forceUpdatePositions = function(game, coinsToPutBack, posList, cb) {
  const gameKey = game.getId();
  logger.debug({key: gameKey}, 'PosList to put coins back is ', posList);
  // if no coin to put back just call the callback
  if (coinsToPutBack.length === 0) return cb(null, null);

  if (coinsToPutBack.length !== posList.length)
    logger.error({key: gameKey}, 'Poslist doesn\'t match with coins to put back list');

  let objTransList = [];
  // update coins position, velocity, force and rotation
  for (let i = 0; i < coinsToPutBack.length; i++) {
    let coin = coinsToPutBack[i];
    let posObj = posList[i];
    let objTrans = {
      id: coin.id,
      posX: posObj.posX,
      posY: posObj.posY,
      posZ: posObj.posZ,
    };
    objTransList.push(objTrans);
  }
  cb(null, () => {
    logger.info({key: gameKey}, 'Force updating coins position to put back,',
        JSON.stringify(objTransList));
    game.forceUpdateGameObjects(...objTransList);
  });
};

const centeringPositions = async function(game, coinsToCenterCount) {
  let coinsCentred = 0;
  let positionsToPush = [];
  let hand = genericFetchUtils.fetchHand(game);

  for (let i = 0, len = LIST_OF_CENTER_POS.length;
       i < len && coinsCentred < coinsToCenterCount; i++) {
    let posBoundary = LIST_OF_CENTER_POS[i];
    let res = await promisifyCollision(game, posBoundary);
    if (res.length === 0 || (res.length === 1 && res[0] === hand.id)) {
      // found position push it into list and increment count
      positionsToPush.push(posBoundary);
      coinsCentred++;
    }
  }
  return positionsToPush;
};

const promisifyCollision = function(game, boundary) {
  return new Promise((resolve, reject) => {
    game.collidedObjsWithBoundary(boundary, (err, res) => {
      if (err)
        return reject(err);
      resolve(res);
    });
  });
};
